@@ -320,7 +320,7 @@ sub _checkValues
             if ($type eq 'post') {
                 next;
             }
-            if (defined($cattr->{required}) && $cattr->{required}[0] eq 'on' && ((($func eq 'add' || $func eq 'search') && (!@values || $values[0] =~ /^ *$/)) || ($func eq 'modify' && $entryStr =~ /$attr: / && (!@values || $values[0] =~ /^ *$/)))) {
+            if (defined($cattr->{required}) && $cattr->{required}[0] eq 'on' && ((($func eq 'add' || $func eq 'search') && (!@values || $values[0] =~ /^ *$/)) || ($func eq 'modify' && $entryStr =~ /^$attr: /mi && (!@values || $values[0] =~ /^ *$/)))) {
                 $self->_perror("$attr in $dn is required value");
                 ${$errorp} = "$attr is required value" if ref($errorp);
                 $rc = LDAP_CONSTRAINT_VIOLATION;
@@ -498,7 +498,7 @@ sub _getParam
         ($base) = ($dn =~ /($opts->{base})/i);
     }
     if (!$base) {
-        return 1;
+        return undef;
     }
 
     if (!defined($self->{lism}->{check_param})) {
@@ -528,7 +528,7 @@ sub _getParam
                     my @tmpvals = ($entries[$i] =~ /^$attr: (.*)$/gmi);
                     if (@tmpvals) {
                         foreach my $tmpval (@tmpvals) {
-                            if ($tmpval !~ /^ *$/ && !grep(/$tmpval$/i, @values)) {
+                            if ($tmpval !~ /^ *$/ && !grep(/^$tmpval$/i, @values)) {
                                 push(@values, $tmpval);
                             }
                         }
