@@ -162,7 +162,7 @@ sub _objSearch
     my @match_keys = ();
     my $rc = LDAP_SUCCESS;
 
-    if (!defined($oconf->{mode}[0]) || $oconf->{mode}[0] ne 'append') {
+    if (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append') {
 
         # do plugin
         if (defined($oconf->{plugin})) {
@@ -200,7 +200,7 @@ sub _objSearch
             while (my $row = $csv->getline($file)) {
                 my @data = @$row;
                 for (my $i = 0; $i < @data; $i++) {
-                    Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+                     Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
                 }
                 my $entry;
                 my $match = 1;
@@ -311,7 +311,11 @@ sub _objSearch
                 $dn =~ s/;/\\3B/g;
                 $dn =~ s/</\\3C/g;
                 $dn =~ s/>/\\3E/g;
-                $entry = decode('utf8', "dn: $dn\n$entry");
+                my $entrystr = "dn: $dn\n$entry";
+                if (!Encode::is_utf8($entrystr)) {
+                    $entry = decode_utf8($entrystr);
+                }
+                $entry = $entrystr;
 
                 # parse filter
                 if ($self->parseFilter($filter, $entry)) {
@@ -363,7 +367,7 @@ sub _objModify
     my $db;
     my $tmp;
 
-    if (!defined($oconf->{mode}[0]) || $oconf->{mode}[0] ne 'append') {
+    if (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append') {
         if (!$self->_checkFile($oconf->{file}[0])) {
             return LDAP_OPERATIONS_ERROR;
         }
@@ -375,7 +379,7 @@ sub _objModify
     }
     flock($lock, 2);
 
-    if (defined($oconf->{mode}[0]) && $oconf->{mode}[0] eq 'append') {
+    if (defined($oconf->{mode}) && $oconf->{mode}[0] eq 'append') {
         if (!defined($oconf->{db})) {
             if (!open($file, ">> $oconf->{file}[0]")) {
                 $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
@@ -450,7 +454,7 @@ sub _objModify
                     if(@data) {
                         $data[$oconf->{attr}{$oconf->{rdn}[0]}{column}[0]] = $rdn_val;
                         foreach my $attr (keys %{$oconf->{attr}}) {
-                            if (defined($oconf->{attr}{$attr}->{column}) && !$data[$oconf->{attr}{$attr}->{column}[0]]) {
+                            if (defined($oconf->{attr}{$attr}->{column}) && !defined($data[$oconf->{attr}{$attr}->{column}[0]])) {
                                 $data[$oconf->{attr}{$attr}->{column}[0]] = "";
                             }
                         }
@@ -478,7 +482,7 @@ sub _objModify
                 }
                 my $line = $csv->string();
                 if (!Encode::is_utf8($line)) {
-                    $line = decode('utf8', $line);
+                    $line = decode_utf8($line);
                 }
                 if (defined($oconf->{db})) {
                     if ($self->_dbInsert($oconf, $conf->{mbcode}[0] eq 'utf8' ? $line : Encode::from_to($line, 'utf8', $conf->{mbcode}[0]), $timestamp)) {
@@ -525,7 +529,7 @@ sub _objModify
             while (my $row = $csv->getline($tmp)) {
                 my @data = @$row;
                 for (my $i = 0; $i < @data; $i++) {
-                    Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+                    Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
                 }
                 # check the data corresponds to the dn
                 if (!("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" =~ /$valdlm$regex_rdn_val$valdlm/i)) {
@@ -637,7 +641,7 @@ sub _objModify
                 }
                 my $line = $csv->string();
                 if (!Encode::is_utf8($line)) {
-                    $line = decode('utf8', $line);
+                    $line = decode_utf8($line);
                 }
                 print $file encode($conf->{mbcode}[0], $line)."\n";
             }
@@ -655,7 +659,7 @@ sub _objModify
         $self->rollback();
     }
 
-    return ($rc, !$rc && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
+    return ($rc, (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append' || !$rc) && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
 }
 
 =pod
@@ -691,7 +695,7 @@ sub _objAdd
     my $file;
     my $tmp;
 
-    if (!defined($oconf->{mode}[0]) || $oconf->{mode}[0] ne 'append') {
+    if (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append') {
         if (!$self->_checkFile($oconf->{file}[0])) {
             return LDAP_OPERATIONS_ERROR;
         }
@@ -710,7 +714,7 @@ sub _objAdd
         return LDAP_OPERATIONS_ERROR;
     }
 
-    if (defined($oconf->{mode}[0]) && $oconf->{mode}[0] eq 'append') {
+    if (defined($oconf->{mode}) && $oconf->{mode}[0] eq 'append') {
         if (!defined($oconf->{db})) {
             if (!open($file, ">> $oconf->{file}[0]")) {
                 $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
@@ -749,7 +753,7 @@ sub _objAdd
 
             my @data = @$row;
             for (my $i = 0; $i < @data; $i++) {
-                Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+                Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
             }
 
             # check the data correspods to the dn
@@ -848,7 +852,7 @@ sub _objAdd
 
             my $line = $csv->string();
             if (!Encode::is_utf8($line)) {
-                $line = decode('utf8', $line);
+                $line = decode_utf8($line);
             }
             if (defined($oconf->{db})) {
                 if ($self->_dbInsert($oconf, $conf->{mbcode}[0] eq 'utf8' ? $line : Encode::from_to($line, 'utf8', $conf->{mbcode}[0]), $timestamp)) {
@@ -869,7 +873,7 @@ sub _objAdd
         $self->rollback();
     }
 
-    return ($rc, !$rc && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
+    return ($rc, (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append' || !$rc) && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
 }
 
 =pod
@@ -905,7 +909,7 @@ sub _objDelete
     my $file;
     my $tmp;
 
-    if (!defined($oconf->{mode}[0]) || $oconf->{mode}[0] ne 'append') {
+    if (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append') {
         if (!$self->_checkFile($oconf->{file}[0])) {
             return LDAP_OPERATIONS_ERROR;
         }
@@ -917,7 +921,7 @@ sub _objDelete
     }
     flock($lock, 2);
 
-    if (!defined($oconf->{mode}[0]) || $oconf->{mode}[0] ne 'append') {
+    if (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append') {
         if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
             $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
             close($lock);
@@ -948,7 +952,7 @@ sub _objDelete
         while (my $row = $csv->getline($tmp)) {
             my @data = @$row;
             for (my $i = 0; $i < @data; $i++) {
-                Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+                Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
             }
 
             # check the data corresponds to the dn
@@ -1028,7 +1032,7 @@ sub _objDelete
                 }
                 my $line = $csv->string();
                 if (!Encode::is_utf8($line)) {
-                    $line = decode('utf8', $line);
+                    $line = decode_utf8($line);
                 }
                 if (defined($oconf->{db})) {
                     if ($self->_dbInsert($oconf, $conf->{mbcode}[0] eq 'utf8' ? $line : Encode::from_to($line, 'utf8', $conf->{mbcode}[0]), strftime("%Y%m%d%H%M%S", localtime(time))."Z")) {
@@ -1051,7 +1055,7 @@ sub _objDelete
         $self->rollback();
     }
 
-    return ($rc, !$rc && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
+    return ($rc, (!defined($oconf->{mode}) || $oconf->{mode}[0] ne 'append' || !$rc) && defined($conf->{noaudit}) && $conf->{noaudit}[0] eq 'on' ? 'LISM_NO_OPERATION' : '');
 }
 
 sub _getParentRdn
@@ -1086,7 +1090,7 @@ sub _getParentRdn
     while (my $row = $csv->getline($file)) {
         my @data = @$row;
         for (my $i = 0; $i < @data; $i++) {
-            Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+            Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
         }
 
         # check the data corresponds to the object's id
@@ -1108,7 +1112,7 @@ sub _getParentRdn
     while (my $row = $csv->getline($file)) {
         my @data = @$row;
         for (my $i = 0; $i < @data; $i++) {
-            Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+            Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
         }
 
         # check the data corresponds to the object's id
@@ -1164,7 +1168,7 @@ sub _getAttrValues
     while (my $row = $csv->getline($file)) {
         my @data = @$row;
         for (my $i = 0; $i < @data; $i++) {
-            Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+            Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
         }
 
         # check the data corresponds to the object's id
@@ -1266,7 +1270,7 @@ sub _getAttrKeys
 
             my @data = @$row;
             for (my $i = 0; $i < @data; $i++) {
-                Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
+                Encode::is_utf8($data[$i]) ? $data[$i] : Encode::from_to($data[$i], $conf->{mbcode}[0], 'utf8');
             }
 
             my $dvals;
